/* chopper.h 
   
   Author : KoreK
   Date : Mar 8,2004
   Licence : GPL v2 or above (see accompanying LICENSE file)
*/


#ifndef CHOPPER_H
#define CHOPPER_H

#include "wep_data.h"

/* grades are floating point */
typedef float Grade;
typedef Grade guess_stat_t[256];



/* guess_cnt_t are a collection of counters used during the attack */
enum Attacks {
    A_w15,  /* interesting, but sometimes false positive */
    A_s13,  /* limited to 256 IV, so not so much available */
    A_w13_1, 
    A_w13_2, /* good/OK */
    A_w13_3, /* looks good*/
    A_s5_1, /* grows with size */
    A_s5_2, 
    A_s5_3,
    A_w5_1, /* good ? */
    A_w5_2,
    A_w5_3, 
    A_w5_4,
    A_s2, /* too little data, doesn't work, no result/or false result AAMOF */
    A_neg, /* inverted */
    A_4_s13,
    A_4_w5_1,
    A_4_w5_2
/*    A_4_w5_3  does not work abandoned!, have to explain big accretion
		values */
};

#define N_ATTACKS 17



typedef u_short u_counter;
//typedef u_counter counters[N_ATTACKS];

typedef u_counter guess_cnt_t[256][N_ATTACKS];


const float cf1[N_ATTACKS]={0.0f,
			    1.6f,1.5f,1.5f,1.5f,
			    1.1f,1.1f,1.1f,
			    1.0f,1.0f,1.0f,1.0f,
			    0.6f,
			    0.0f,
			    1.6f,
			    1.0f,1.0f};


/* constants */





const Grade RANK_MUL_COEFF=0.01f;
//const Grade FEEDBACK=0.4f; might have been better
//const Grade FEEDBACK=0.6f; not so good
//const Grade FEEDBACK=0.15f; 
const Grade FEEDBACK=0.25f; 
const Grade DIFF_KEY_COEFF=1.5f;
/* M_INF aka minus infinity */
const Grade M_INF=-100000.0f;
// const Grade DIV_F=5.0f;
const Grade DIV_F=5.0f;

const size_t TRUNC=3;
const Grade CULL_COEFF=0.4f;

/* klen is the key length (0 if unknown)
   clen is the current length, gives the index of attacked byte */
struct key {
    Grade gr;
    u_char K[29];
    u_char klen;
    u_char clen;
    u_char pad;
};


/* the first element of the heap is not used !*/
#define MAXHEAPSIZE (1+(1<<15)) 
struct heap_key {
    size_t max_size;
    size_t s;
    
    size_t idx[MAXHEAPSIZE];
    struct key k[MAXHEAPSIZE];
};



typedef void (*eval_guess)(struct wep_data *w,struct key *k,guess_cnt_t g);


void print_constants();


/* heap functions */
void heapsort(struct heap_key *H);
void remove_heap_dup(struct heap_key *H);
void rank_heap(struct heap_key *H);
void truncate_heap(struct heap_key *H);
void cull_heap(struct heap_key *H);
void insert_grade(struct heap_key *H,const struct key *oldK,
		  const guess_stat_t g);
void inc_clen(struct heap_key *H);
void set_clen(struct heap_key *H,size_t clen);
void swap_heaps(struct heap_key **c,struct heap_key **o);
void print_heap(FILE *stream,struct heap_key *H);

/* heap function helpers */
void swap_heap_el(struct heap_key *H,size_t i,size_t j);
void heapify_up(struct heap_key *H,size_t i);
void heapify_down(struct heap_key *H,size_t i,size_t s);
void insertnewkey(struct heap_key *H,const struct key *K);


/* key functions */
void print_key(FILE *stream,const char *format,const struct key *k);
Grade diff_key(const struct key *k1,const struct key *k2);

/* guess manipulation */
void reset_guess(guess_cnt_t g);
void est_guess(struct key *K,guess_cnt_t gc,guess_stat_t gs);
void adj_guess(guess_stat_t gs);
/* size_t max_guess(guess_stat_t gs); */
void print_counters(guess_cnt_t g,guess_stat_t gs);

/* attack function and helpers */
void plain_attack(struct wep_data *w,struct key *k,guess_cnt_t g);
void attack_last(struct wep_data *w,struct key *k,guess_cnt_t g);

/* init a permutation */
void initS(u_char *S);

inline void swap(unsigned char *s1,unsigned char *s2) 
{
    unsigned char sw;
    sw=*s1;
    *s1=*s2;
    *s2=sw;
}

Grade check_key(struct wep_data *w,struct key *k,size_t inc);

#endif

