/* gen.c
   
   Author : KoreK
   Date : Mar 8,2004
   Licence : GPL v2 or above (see accompanying LICENSE file)
*/
#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <limits.h>
#include <unistd.h>
#include "rg.h"
#include "wep_data.h"
#include "gen.h"


int init_wepdata(struct wep_data *w,size_t n_samples,size_t key_len,
		 unsigned char *K,unsigned int p,unsigned int J_present)
{

    if ((J_present!=0) && (J_present!=1)) return INVPAR;

    w->p=p;
    w->J_present=J_present;

    w->key_len=key_len;
    if (key_len!=0) {
	w->key=(unsigned char *)malloc(key_len);
	if (K!=0) memcpy(w->key,K,key_len);
    }
    else {
	w->key=0;
    }

    w->n_samples=n_samples;
    w->sample_size=3+p*(1+2*J_present);

    if (n_samples != 0) 
	w->sample=(unsigned char *)malloc(n_samples*w->sample_size);
    else 
	w->sample=0;

    w->j_table=0;
    w->S_table=0;
    w->res_i=0;

    return 0;
}


int set_random_key(struct wep_data *w,struct random_gen *r)
{
    size_t i;
    for (i=0;i<w->key_len; i++) {
	w->key[i]=next_byte(r);
    }
    return 0;
}


void delete_tables(struct wep_data *w)
{
    if (w->j_table != 0) {
	free(w->j_table);
	w->j_table=0;
    }
    if (w->S_table != 0) {
	free(w->S_table);
	w->S_table=0;
    }
}


void delete_reson_idx(struct wep_data *w)
{
    if (w->res_i != 0) {
	free(w->res_i);
	w->res_i=0;
    }
}



void delete_wepdata(struct wep_data *w)
{
    if (w->sample!=0) free(w->sample);
    w->sample=0;

    if (w->key!=0) free(w->key);
    w->key=0;
    delete_reson_idx(w);
    delete_tables(w);
}


int allIV(struct wep_data *w)
{
    size_t acnt,cnt,fs,p;
    unsigned int J_present,key_len;
    unsigned char *K,*sample;

//    size_t cur,cur_m;
//    unsigned char *b;

    size_t i,is;
//    size_t idx,idx_m;
    unsigned char S[256];
    unsigned char sw,j,t;

    acnt=w->n_samples;
    J_present=w->J_present;
    p=w->p;
    fs=3+(1+2*J_present)*p;
    sample=w->sample;
    key_len=w->key_len+3;
    
    if (key_len==3) return NOKEY;
    if (acnt == 0) return NOIV;

    K=(unsigned char *)malloc(key_len);
    if (K==0) return ERRALLOC;
    for (i=3; i<key_len; i++) K[i]=w->key[i-3];

    for (cnt=0; cnt<acnt; cnt++) {
	    K[0]=cnt & 0xff;
	    K[1]=(cnt >> 8) & 0xff;
	    K[2]=(cnt >> 16) & 0xff;

	    for (i=0; i<256; i++) S[i]=i;
	    j=0;

	    /* KSA */
	    for (i=0; i<3; i++,sample++) {
		j=j+K[i]+S[i];
		/* reverted to *sample=K[i] 
		   *sample=j; */
		*sample=K[i];
		sw=S[i];
		S[i]=S[j];
		S[j]=sw;
	    }
	    for (is=3; i<256; i++,is++) {
		is=(is==key_len)? 0 : is;
		j=j+K[is]+S[i];
		sw=S[i];
		S[i]=S[j];
		S[j]=sw;
	    }

	    /* PRGA */
	    j=0;
	    for (i=1; i<=p; i++, sample++) {
		j=j+S[i];
		t=S[i]+S[j];
		
		if (J_present==1) {
		    sample[p]=j;
		    sample[2*p]=S[j];
		}

		sw=S[i];
		S[i]=S[j];
		S[j]=sw;
		*sample=S[t];
	    }
	    if (J_present==1) sample=sample+2*p;
    }

    free(K);
    return 0;
}


int fillIV(struct wep_data *w,struct random_gen *r)
{
    size_t cnt,fs,p;
    unsigned int J_present,key_len;
    unsigned char *K,*sample;

    size_t cur,cur_m;
    unsigned char *b;

    size_t i,is,idx,idx_m;
    unsigned int *done;
    unsigned char S[256];
    unsigned char sw,j,t;

    cnt=w->n_samples;
    J_present=w->J_present;
    p=w->p;
    fs=3+(1+2*J_present)*p;
    sample=w->sample;
    key_len=w->key_len+3;
    
    if (key_len==3) return NOKEY;
    if (cnt > LIMIT) return TOOMANYIVS;
    if (cnt == 0) return NOIV;

    done=(unsigned int *)malloc(1 << 21);
    if (done==0) return ERRALLOC;

    K=(unsigned char *)malloc(key_len);
    if (K==0) {
	free(done);
	return ERRALLOC;
    }
    for (i=3; i<key_len; i++) K[i]=w->key[i-3];

    b=(unsigned char *)r->b;
    cur=r->cur;
    cur_m=r->bs-r->cur-2;

    while (cnt!=0) {
	while ((cur<cur_m) && (cnt!=0)) {
	    K[0]=b[cur++];
	    K[1]=b[cur++];
	    K[2]=b[cur++];
	    idx=(K[0]<<16) + (K[1]<<8) + K[2];
	    idx_m=1 << (idx & 0x1f);
	    idx=idx>>5;
	    if ((done[idx] & idx_m ) == 0) { 
		done[idx]=done[idx] | idx_m;
		cnt--;
	       
		for (i=0; i<256; i++) S[i]=i;
		j=0;
		/* KSA */
		for (i=0; i<3; i++,sample++) {
		    j=j+K[i]+S[i];
		    /* reverted to *sample=K[i] 
		       *sample=j; */
		    *sample=K[i];
		    sw=S[i];
		    S[i]=S[j];
		    S[j]=sw;
		}
		for (is=3; i<256; i++,is++) {
		    is=(is==key_len)? 0 : is;
		    j=j+K[is]+S[i];
		    sw=S[i];
		    S[i]=S[j];
		    S[j]=sw;
		}

		/* PRGA */
		j=0;
		for (i=1; i<=p; i++, sample++) {
		    j=j+S[i];
		    t=S[i]+S[j];

		    if (J_present==1) {
			sample[p]=j;
			sample[2*p]=S[j];
		    }

		    sw=S[i];
		    S[i]=S[j];
		    S[j]=sw;
		    *sample=S[t];
		}
		if (J_present==1) sample=sample+2*p;
	    }

	}
	update_rg(r);
	cur=0;
	cur_m=r->bs-2; 
    }

    r->cur=cur;
    free(done);
    free(K);
    return 0;
}




int write_wep(struct wep_data *w,const char *fn)
{
    FILE *f;
    size_t header[6];
    
    /* check if w valid */
    if (w->sample == 0) return -1;

    f=fopen(fn,"w");
    if (f==0) return -1;
    
    header[0]=WEP_MAGIC;
    header[1]=w->n_samples;
    header[2]=w->p;
    header[3]=w->J_present;
    header[4]=w->key_len;
    header[5]=w->sample_size;
    
    
    if (fwrite(header,sizeof(size_t),6,f)!=6) {
	fclose(f);
	unlink(fn);
	return -1;
    }

    if (fwrite(w->sample,w->sample_size,w->n_samples,f)!=w->n_samples) {
	fclose(f);
	unlink(fn);
	return -1;
    }

    if (w->key_len!=0) {
	if (fwrite(w->key,w->key_len,1,f)!=1) {
	    fclose(f);
	    unlink(fn);
	    return -1;
	}
    }
    fclose(f);

    return 0;
}



int read_wep(struct wep_data *w,const char *fn)
{
    FILE *f;
    size_t header[6];
    long fs;

    delete_wepdata(w);
    init_wepdata(w,0,0,0,0,0);

    f=fopen(fn,"r");
    if (f==0) return -1;

    if (fseek(f,0,SEEK_END)) return -1;
    fs=ftell(f);
    if (fseek(f,0,SEEK_SET)) return -1;

    if (fread(header,sizeof(size_t),6,f)!=6) {
	fclose(f);
	return -1;
    }

    if (header[0]==WEP_MAGIC) {
	w->n_samples=header[1];
	w->p=header[2];
	w->J_present=header[3];
	w->key_len=header[4];
	w->sample_size=header[5];
	
    } else if ((header[0]==RAW_MAGIC) && (((fs-4)%5)==0)) {
	w->n_samples=(unsigned )(fs-4)/5;
	w->p=2;
	w->J_present=0;
	w->key_len=0;
	w->sample_size=5;

	fseek(f,4,SEEK_SET);

    } else {
	fclose(f);
	return -2;
    }

    /* allocate the memory */
    w->sample=calloc(w->n_samples,w->sample_size);
    if (w->sample==0) {
	fclose(f);
	return -1;
    }
    
    if (fread(w->sample,w->sample_size,w->n_samples,f)!=w->n_samples) {
	fclose(f);
	delete_wepdata(w);
	return -1;
    }
	
    if (w->key_len!=0) {
	w->key=calloc(w->key_len,1);
	if (w->key==0) {
	    fclose(f);
	    delete_wepdata(w);
	    return -1;
	}
	if (fread(w->key,w->key_len,1,f)!=1) {
	    fclose(f);
	    delete_wepdata(w);
	    return -1;
	}
    }

    if (header[0]==RAW_MAGIC) {
	size_t i;
	u_char *p;
	p=w->sample;
	for (i=0; i<w->n_samples;i++) {
	    p[3]^=0xaa;
	    p[4]^=0xaa;
	    p+=5;
	}
    }

    fclose(f);
    return 0;

}    
    
