/* 
   Author : KoreK
   Date : Feb 2,2004
   Licence : GPL v2 or above (see accompanying LICENSE file)
*/


/* Implementation of the Mersenne twister following the article :
   Mersenne Twister: A 623-dimensionally equidistributed uniform 
   pseudorandom number generator

   By Makoto Matsumoto and Takuji Nishimura

   Ref: ..../mt.ps
*/

#include <stdlib.h>
#include <string.h>
#include <limits.h>
#include "rg.h"



/* Mersenne twister constants */
#define N 624
#define M 397
#define A 0x9908b0df
#define R 31
#define LOWERM ((1<<R)-1)
#define UPPERM (~LOWERM)
/* tempering parameters */
#define T_U 11
#define T_S 7
#define T_T 15
#define T_L 18
#define T_B 0x9d2c5680
#define T_C 0xefc60000

/* bit mask to round up to a multiple of 4096, ie UINT_MAX with 
   the last twelve bits cleared */
#define BM4096 (UINT_MAX << 12)
#define SEEDSIZE (N*4)


char mt_alg_name[]="Mersenne twister";

size_t mt_buffer_size_ra(size_t b);
int mt_init_ra_rg(struct random_gen *rg,struct random_gen *s);
int mt_init_ra_seed(struct random_gen *rg,void *data,size_t len);
void mt_update(struct random_gen *rg);

struct random_algo mt_random[1]={{mt_alg_name, 
				  SEEDSIZE, SEEDSIZE, 
				  &mt_buffer_size_ra, 
				  &mt_init_ra_rg, &mt_init_ra_seed, 
				  0, 0, 
				  &mt_update
#ifdef MAGIC
				  , MAGIC_RESET 
#endif
}};



size_t mt_buffer_size_ra(size_t b)
{
    return ((b-1) & 4096) + 4096;
}

int mt_init_ra_rg(struct random_gen *rg,struct random_gen *s)
{
    void *buf;

    buf=rg->b + (rg->bs - SEEDSIZE);
    fill_rand_buff(buf,s,SEEDSIZE);

    mt_update(rg);
    rg->cur=0;
    return 0;
}

int mt_init_ra_seed(struct random_gen *rg,void *data,size_t len)
{
    void *buf;

    if (len<SEEDSIZE) return SEED_SHORT;

    buf=rg->b + (rg->bs - SEEDSIZE);
    memcpy(buf,data,SEEDSIZE);

    mt_update(rg);
    rg->cur=0;
    return 0;
}


void mt_update(struct random_gen *r)
{
    unsigned int *x;
    int i;
    size_t buf_mask,buf_size;
    unsigned int r0,r1,r2,r3,r4,r5;

    x=(unsigned int *)r->b;
    buf_size=r->bs >> 2;
    buf_mask=buf_size - 1;

    for (i=0; i<buf_size; i++) {
	
	r0=(x[(i-N) & buf_mask] & UPPERM) | (x[(i+1-N) & buf_mask] & LOWERM);
	r1=(r0 >> 1) ^ ((-(r0 & 1)) & A); 
	r2=x[(i-N+M) & buf_mask] ^ r1;
	
	r3=r2 ^ (r2 >> T_U);
	r4=r3 ^ ((r3 << T_S) & T_B);
	r5=r4 ^ ((r4 << T_T) & T_C);
	x[i]=r5 ^ (r5 >> T_L);
    }
}

