/* rg.c: 
   glue code for random generators 
*/

/* 
   Author : KoreK
   Date : Feb 1,2004
   Licence : GPL v2 or above (see accompanying LICENSE file)
*/

/* Weaknesses: 
   Error codes are not correctly implemented
   Magic not implemented
*/

#ifdef MAGIC
#include <stdio.h>
#include <debug.h>
#include "magic.h"
#endif

#include <stdlib.h>
#include <string.h>
#include "rg.h"


int create_rg(struct random_gen *rg,const struct random_algo *ra,size_t bs)
{
#ifdef MAGIC
#ifdef MEM_LEAK
    if (rg->magic == RANDOM_GEN_MAGIC) 
	debug("MEM_LEAK: overwriting rg",MEM_LEAK);
#endif
    if (ra->magic != RANDOM_ALG_MAGIC)
	debug("Unitialized random_algo in init_rg",-1);
#endif

    if (ra->setup_ra_rg !=0) {
	return ra->setup_ra_rg(rg);
    }
    else {
	rg->bs=ra->buffer_size_ra(bs);
	rg->b=malloc(rg->bs);
	if (rg->b == 0) {
/*	    RESET_MAGIC(rg->magic); */
	    return MALLOC_ERR;
	}
	rg->cur=0;    
	rg->update=ra->update;
	rg->ra=ra;

/*      SET_MAGIC(rg->magic,RANDOM_GEN_MAGIC); */
	return 0;
    }
}


size_t seed_size(const struct random_algo *ra)
{
    return ra->min_seed_size;
}


int init_rg(struct random_gen *rg,struct random_gen *init)
{
    if (rg->ra->init_ra_rg != 0) {
	if (rg != init) return rg->ra->init_ra_rg(rg,init);
	else return INIT_ERROR;
    }
    else return NOT_AVAIL;
}


int init_rg_seed(struct random_gen *rg,void *data,size_t len)
{
    if (len < rg->ra->min_seed_size) return SEED_SHORT;
    else return rg->ra->init_ra_seed(rg,data,len);
}


int delete_rg(struct random_gen *rg)
{
    if (rg->ra->clean_ra_rg != 0) {
	return rg->ra->clean_ra_rg(rg);
    }
    else {
	free(rg->b);
	rg->cur=0;
	rg->ra=0;
	return 0;
    }
}


int fill_rand_buff(void *buff,struct random_gen *rg,size_t len)
{
    size_t off,av;

    if (rg->cur >= rg->bs) update_rg(rg);

    off=0;
    av=rg->bs - rg->cur;

    while (av<len-off) {
	memcpy(buff+off,rg->b+rg->cur,av);
	off=off+av;
	update_rg(rg);
	av=rg->bs;
    }

    memcpy(buff+off,rg->b+rg->cur,len-off);

    return 0;
}

