/* rg.h:
   Header file for random generators
 */

/*
   Use:
     rando_algo_t ra;
     rando_gen_t rg,init_gen;
     size_t buffer_size;
     
     create_rg(rg,ra,buffer_size);
     init_rg(rg,init_gen);
     ...
     next_byte(rg);
     ...
     delete_rg(rg);
*/

/* 
   Author : KoreK
   Date : Feb 1,2004
   Licence : GPL v2 or above (see accompanying LICENSE file)
*/

#ifndef RG_H
#define RG_H

#ifdef MAGIC
#include "magic.h"
#endif
#include <unistd.h>

struct random_gen;


/* important note: setup_ra_rg and clean_ra_rg should be both NULL
   or both non-NULL.
*/
struct random_algo {
    char *name;
    size_t min_seed_size,max_seed_size;

    size_t (*buffer_size_ra)(size_t size);
    int (*init_ra_rg)(struct random_gen *rg,struct random_gen *s);
    int (*init_ra_seed)(struct random_gen *rg,void *data,size_t len);
    int (*setup_ra_rg)(struct random_gen *rg);
    int (*clean_ra_rg)(struct random_gen *rg);
    void (*update)(struct random_gen *rg);

#ifdef MAGIC
    int magic;
#endif
};

struct random_gen {
    void *b;
    size_t bs,cur;
    void (*update)(struct random_gen *rg);

    const struct random_algo *ra;
#ifdef MAGIC
    int magic;
#endif
};

typedef struct random_gen random_gen_t[1];


/* error code */
#define INIT_ERROR 1 
#define NOT_AVAIL 2
#define SEED_SHORT 3
#define MALLOC_ERR 4

/* interface */

extern int create_rg(struct random_gen *rg,const struct random_algo *ra,size_t bs);

extern size_t seed_size(const struct random_algo *ra);

extern int init_rg(struct random_gen *rg,struct random_gen *init);

extern int init_rg_seed(struct random_gen *reg,void *data,size_t len);


static inline void update_rg(struct random_gen *rg)
{
    rg->update(rg);
    rg->cur=0;
}

static inline unsigned char next_byte(struct random_gen *rg)
{
    if (rg->cur > rg->bs) {
	update_rg(rg);
	rg->cur=0;
    }
    return ((char *)rg->b)[rg->cur++];
}


extern int delete_rg(struct random_gen *rg);



/* useful functions */
extern int fill_rand_buff(void *buff,struct random_gen *rg,size_t len);


/* various random generator */
extern struct random_algo std_random[1];
extern struct random_algo mt_random[1];

#endif 
