/* std_rg.c
   
   Author : KoreK
   Date : Mar 8,2004
   Licence : GPL v2 or above (see accompanying LICENSE file)
*/

#include <stdlib.h>
#include <limits.h>
#include "rg.h"


/* bit mask to round up to a multiple of 4, ie UINT_MAX with 
   the last two bits cleared */
#define BM4 (UINT_MAX ^ 3)

char alg_name[]="Standard rand";

size_t buffer_size_ra(size_t b);
int init_ra_rg(struct random_gen *rg,struct random_gen *s);
int init_ra_seed(struct random_gen *rg,void *data,size_t len);
void update(struct random_gen *rg);


struct random_algo std_random[1]={{alg_name, 
				   0, 4, 
				   &buffer_size_ra, 
				   &init_ra_rg, &init_ra_seed, 
				   0, 0, 
				   &update
#ifdef MAGIC
				   , MAGIC_RESET 
#endif
				   }};



size_t buffer_size_ra(size_t b)
{
    return ((b-1) & BM4) + 4;
}

int init_ra_rg(struct random_gen *rg,struct random_gen *s)
{
    unsigned int seed;
    fill_rand_buff(&seed,s,sizeof(unsigned int));
    srand(seed);
    return 0;
}

int init_ra_seed(struct random_gen *rg,void *data,size_t len)
{
    unsigned int i,seed=0;
    
    if (len > 4) len=4;

    if ((data != 0) && (len != 0)) {
	for (i=0; i<len; i++) {
	    seed<<=8;
	    seed+=((char *)data)[i];
	}
    }
    else seed=1;

    srand(seed);
    update(rg);
    rg->cur=0;
    return 0;
}

void update(struct random_gen *rg)
{
    unsigned int *b; 
    size_t i,N;
    unsigned int acc,nb;
    
    b=(unsigned int *)rg->b;
    N=rg->bs >> 2;
    acc=random();
    nb=31;

    for (i=0; i<N; i++) {
	if (nb != 0) {
	    b[i]=acc;
	}
	else
	{
	    b[i]=random();
	    nb=31;
	}

	acc=random();
	b[i]=b[i] | (acc << nb);
	acc=acc >> (32-nb);
	nb--;
    }
}
