/* wep_gen.c
   
   Author : KoreK
   Date : Mar 8,2004
   Licence : GPL v2 or above (see accompanying LICENSE file)
*/

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include "rg.h"
#include "wep_data.h"
#include "gen.h"

const char usage[]="\
wep_gen <nfiles> <nsamples> <nels> <key_len> <rg> <seed>\n\
first four arguments are mandatory\n\
nfiles: number of files to be generated ( less or eq than 10000)\n\
nsamples: number of samples per key (understand k for kilo)\n\
nels: number of bytes of the output stream saved\n\
key_len: len of the key (less than 32)\n\
rg: 0 (standard) 1 (mt)\n\
seed: long (by default /dev/urandom) used to seed whatever gen\n";



/* wep_gen creates data sets to be used for analysis (I guess)
   arguments
   1) n_samples order, ie n_samples=1<<$1
   2) number of elements
   3) key_len
   4) random generator
   5) seed / seeded by urandom by default

   the output file is key-<4digitseq>-<hex-seed>.<hex-key>
   it does up to 10000 iters, but 
*/


int main(int argv,char **argc)
{


    char *cptr;
    size_t n_samples,nels,key_len,rgn,n_files;
    unsigned int seed; 
    char seed_str[11];
    char key_str[65],pad[3];
    /* 64 (key) + 4 (number 0000-9999) + 10 (rg+seed) + 
       6 (key- - -) + 1 (null) = 87 */
    char fn[87];
    const char fn_format[]="key-%s-%04u-%s";
    
    wep_data_t w;
    random_gen_t rg;
    size_t i,j,js;
    



    if (argv<3) {
	fprintf(stderr,usage);
	return -1;
	
    }
    cptr=0;
    n_files=strtol(argc[1],0,0);

    n_samples=strtol(argc[2],&cptr,0);
    if (*cptr=='k') n_samples=n_samples*1024;

    nels=strtol(argc[3],0,0);
    key_len=strtol(argc[4],0,0);
    
    if (argv>5) {
	rgn=strtol(argc[5],0,0);
    }
    else {
	rgn=0;
    }

    if (argv>6) {
	seed=strtol(argc[6],0,0);
    }
    else {
	int fd;
	
	fd=open("/dev/urandom",O_RDONLY);
	if (fd==-1) {
	    fprintf(stderr,"Could not open random device\n");
	    return -1;
	}
	if (read(fd,&seed,4)!=4) {
	    fprintf(stderr,"Failed proper urandom read\n");
	    close(fd);
	    return -1;
	}
	close(fd);
    }

    if ((n_samples>=LIMIT) || (nels>4) || (key_len==0) ||
	(key_len>32) || (rgn>1) || (n_files>10000)) {
	fprintf(stderr,"parameter out of bound\n");
	return -1;
    }

    init_wepdata(w,n_samples,key_len,0,nels,0);
    snprintf(seed_str,11,"%1d-%08x",rgn,seed);
    printf("Seed : %s\n",seed_str);
    
    if (rgn==0) {
	create_rg(rg,std_random,4096);
	init_rg_seed(rg,&seed,4);
	update_rg(rg);
    }
    else {
	random_gen_t rg_r;
	create_rg(rg_r,std_random,1024);
	init_rg_seed(rg_r,&seed,4);

	create_rg(rg,mt_random,4096);
	init_rg(rg,rg_r);
	update_rg(rg);

	delete_rg(rg_r);
    }
    
    for (i=0; i<n_files; i++) {
	set_random_key(w,rg);

	for (j=0,js=0; j<key_len; j++) {
	    snprintf(pad,3,"%02x",w->key[j]);
	    key_str[js++]=pad[0];
	    key_str[js++]=pad[1];
	}
	key_str[js]=0;
	snprintf(fn,87,fn_format,seed_str,i,key_str);

	printf("Key: %s\n",key_str);
	fflush(stdout);

	fillIV(w,rg);
	
	if (write_wep(w,fn)!=0) {
	    fprintf(stderr,"could not write file %s\n",fn);
	    return -1;
	}
    }
    
    delete_wepdata(w);
    delete_rg(rg);
    return 0;

}
